/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioIndiceIRKBean;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

/**
 * @author valdevir_rj
 * 
 */
public class RelatorioIndiceIRK extends Relatorio {

	private List<RelatorioIndiceIRKBean> lsDadosRelatorio;
	private static Logger log = Logger.getLogger(RelatorioIndiceIRK.class);
	

	/**
	 * @param parametros
	 * @param conexao
	 */
	public RelatorioIndiceIRK(final Map<String, Object> parametros, final Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {
				try {
					Integer rutaId = parametros.get("RUTA_ID") != null ? Integer.valueOf(parametros.get("RUTA_ID").toString()) : null;
					Empresa empresa = parametros.get("EMPRESA") != null ? (Empresa) (parametros.get("EMPRESA")) : null;
					Integer grupoRutaId = parametros.get("GRUPORUTA_ID") != null ? Integer.valueOf(parametros.get("GRUPORUTA_ID").toString()) : null;
					String rutaIds = parametros.get("LINHA_FILTRO") != null ? (parametros.get("LINHA_FILTRO").toString()) : null;
					String corridasIds = parametros.get("SERVICO_FILTRO") != null ? (parametros.get("SERVICO_FILTRO").toString()) : null;

					// Executa query relatrio e preenche objeto relatrio
					lsDadosRelatorio= preencheAgrupaRelatorioBean(preparaExecutaQuery(parametros, conexao, rutaId, empresa, grupoRutaId, rutaIds, corridasIds));

					// Ordena pela Desc Linha
					Collections.sort(lsDadosRelatorio, new Comparator<RelatorioIndiceIRKBean>() {
						@Override
						public int compare(RelatorioIndiceIRKBean rel1, RelatorioIndiceIRKBean rel2) {
							return rel1.getDescRuta().compareTo(rel2.getDescRuta());
						}
					});

					parametros.put("RELATORIOLIST", lsDadosRelatorio);
					setLsDadosRelatorio(lsDadosRelatorio);
				} catch (Exception e) {
					log.error("Erro ao executar relatrio", e);
				}

			}

			/**
			 * @param parametros
			 * @param conexao
			 * @param rutaId
			 * @param empresa
			 * @param grupoRutaId
			 * @return
			 * @throws SQLException
			 */
			private ResultSet preparaExecutaQuery(final Map<String, Object> parametros, final Connection conexao, Integer rutaId, Empresa empresa, Integer grupoRutaId,String rutaIds, String corridasIds) throws SQLException {
				String sql = getSql(rutaId, grupoRutaId, rutaIds, corridasIds);
				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);
				ResultSet rset = null;
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
				if (parametros.get("DATA_INICIO") != null) {
					Date dataInicio = (Date) parametros.get("DATA_INICIO");
					stmt.setString("DATA_INICIO", sdf.format(dataInicio));
				}

				if (parametros.get("DATA_FINAL") != null) {
					Date dataFinal = (Date) parametros.get("DATA_FINAL");
					stmt.setString("DATA_FINAL", sdf.format(dataFinal));
				}
				stmt.setInt("empresaId", empresa.getEmpresaId());
				rset = stmt.executeQuery();
				return rset;
			}

		});
	}

	/**
	 * @param lsDadosRelatorio
	 */
	public void setLsDadosRelatorio(List<RelatorioIndiceIRKBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	public void processaParametros() throws Exception {
	}

	private String getSql(Integer rutaId, Integer grupoRutaId, String rutaIds, String corridasIds) {
		String TODOS = "TODOS";
		StringBuilder sql = new StringBuilder();
		sql.append(" SELECT ");
		sql.append(" servico,");
		sql.append(" rutaid,");
		sql.append(" descruta,");
		sql.append(" sentido,");
		sql.append(" gruporuta,");
		sql.append(" indiceirkminimo,");
		sql.append(" hora,");
		sql.append(" extensao,");
		sql.append(" SUM(seguro)                                                        AS seguro,");
		sql.append(" SUM(txembarque)                                                    AS txembarque,");
		sql.append(" SUM(pedagio)                                                       AS pedagio,");
		sql.append(" SUM(passagens)                                                     AS passagens,");
		sql.append(" ( SUM(passagens) + SUM(pedagio) + SUM(txembarque) + SUM(seguro) )  AS totalhorario,");
		sql.append(" round(((SUM(passagens) + SUM(pedagio) + SUM(txembarque) + SUM(seguro)) / extensao), 2) AS irk, ");
		sql.append(" indiceirkminimo * extensao                                         AS receitaminima,");
		sql.append(" (SUM(passagens) + SUM(pedagio) + SUM(txembarque) + SUM(seguro) ) - ( indiceirkminimo * extensao )                AS diferencareceita ");
		sql.append(" FROM ");
		sql.append(" ( ");
		sql.append(" SELECT ");
		sql.append(" cj.caja_id                                AS cajaId,  ");
		sql.append(" cj.corrida_id                             AS servico, ");
		sql.append(" r.ruta_id                                 AS rutaid, ");
		sql.append(" r.descruta                                AS descruta, ");
		sql.append(" r.indsentidoida                           AS sentido, ");
		sql.append(" nvl(e.indiceirk, 0)                       AS indiceirkminimo, ");
		sql.append(" nvl(gr.descgrupo, 'No Definido')         AS gruporuta, ");
		sql.append(" to_char(cj.fechorviaje, 'HH24:MI')        AS hora, ");
		sql.append(" tr.cantkmreal                             AS extensao, ");
		sql.append(" cj.importeseguro                          AS seguro, ");
		sql.append(" cj.importetaxaembarque                    AS txembarque, ");
		sql.append(" cj.importepedagio                         AS pedagio, ");
		sql.append(" cj.preciopagado                           AS passagens ");
		sql.append(" FROM ");
		sql.append(" caja cj ");
		sql.append(" INNER JOIN ruta              r ON ( cj.ruta_id = r.ruta_id ) ");
		sql.append(" LEFT JOIN grupo_ruta        gr ON ( r.gruporuta_id = gr.gruporuta_id ) ");
		sql.append(" INNER JOIN ruta_combinacion  rc ON ( rc.ruta_id = cj.ruta_id ");
		sql.append(" AND rc.activo = 1 ) ");
		sql.append(" INNER JOIN tramo             tr ON ( tr.tramo_id = rc.tramo_id ");
		sql.append(" AND tr.origen_id = cj.origen_id ");
		sql.append(" AND tr.destino_id = cj.destino_id ) ");
		sql.append(" INNER JOIN marca m ON ( cj.marca_id = m.marca_id ");
		sql.append(" AND m.empresa_id = :empresaId  ) ");
		sql.append(" INNER JOIN empresa e ON ( e.empresa_id = :empresaId ) ");
		sql.append(" WHERE ");
		sql.append(" cj.activo = 1 ");
		sql.append(" AND cj.motivocancelacion_id IS NULL ");
		sql.append(" AND cj.feccorrida BETWEEN TO_DATE(:DATA_INICIO, 'DD/MM/YY hh24:mi:ss') AND TO_DATE(:DATA_FINAL, 'DD/MM/YY hh24:mi:ss') ");
		sql.append(rutaIds == null ||TODOS.equals(rutaIds) ? "" : "   AND    r.ruta_id IN (   " + rutaIds + "   )     ");
		sql.append(corridasIds == null || TODOS.equals(corridasIds)	 ? "" : "   AND    cj.corrida_id IN (   " + corridasIds + "   )     ");
		sql.append(grupoRutaId == null ? "" : "   AND    R.GRUPORUTA_ID IN (   " + grupoRutaId + "   )     ");
		sql.append(" GROUP BY ");
		sql.append(" cj.caja_id, ");
		sql.append(" cj.corrida_id, ");
		sql.append(" r.ruta_id, ");
		sql.append(" r.descruta, ");
		sql.append(" r.indsentidoida,");
		sql.append(" e.indiceirk, ");
		sql.append(" gr.descgrupo, ");
		sql.append(" cj.fechorviaje, ");
		sql.append(" tr.cantkmreal, ");
		sql.append(" cj.importeseguro, ");
		sql.append(" cj.importetaxaembarque, ");
		sql.append(" cj.importepedagio, ");
		sql.append(" cj.preciopagado ");
		sql.append(" ORDER BY ");
		sql.append(" r.descruta ");
		sql.append(" ) dados_sem_agrupamento ");
		sql.append(" GROUP BY ");
		sql.append(" servico, ");
		sql.append(" rutaid, ");
		sql.append(" descruta, ");
		sql.append(" gruporuta, ");
		sql.append(" sentido, ");
		sql.append(" indiceirkminimo, ");
		sql.append(" hora, ");
		sql.append(" extensao ");
		sql.append(" order by hora ");
		return sql.toString();

	}

	private List<RelatorioIndiceIRKBean> preencheAgrupaRelatorioBean(ResultSet rset) throws SQLException {
		lsDadosRelatorio = new ArrayList<RelatorioIndiceIRKBean>();
		while (rset.next()) {
			RelatorioIndiceIRKBean relatorioIndiceIRK = new RelatorioIndiceIRKBean();
			relatorioIndiceIRK.setServico((Integer) rset.getInt("servico"));
			relatorioIndiceIRK.setRutaId((Integer) rset.getInt("rutaid"));
			relatorioIndiceIRK.setDescRuta((String) rset.getObject("descruta"));
			relatorioIndiceIRK.setGrupoRuta((String) rset.getObject("gruporuta"));
			relatorioIndiceIRK.setSentido(Boolean.TRUE.equals((Boolean) rset.getBoolean("sentido")) ? "Ida" : "Volta");
			relatorioIndiceIRK.setIrkMinimo((BigDecimal) rset.getObject("indiceirkminimo"));
			relatorioIndiceIRK.setHora((String) rset.getObject("hora"));
			relatorioIndiceIRK.setExtensao((BigDecimal) rset.getObject("extensao"));
			relatorioIndiceIRK.setKmRodado((BigDecimal) rset.getObject("extensao"));
			relatorioIndiceIRK.setSeguro((BigDecimal) rset.getObject("seguro"));
			relatorioIndiceIRK.setTxEmbarque((BigDecimal) rset.getObject("txembarque"));
			relatorioIndiceIRK.setPedagio((BigDecimal) rset.getObject("pedagio"));
			relatorioIndiceIRK.setPassagens((BigDecimal) rset.getObject("passagens"));
			relatorioIndiceIRK.setTotalViagem((BigDecimal) rset.getObject("totalhorario"));
			relatorioIndiceIRK.setTotal((BigDecimal) rset.getObject("totalhorario"));
			relatorioIndiceIRK.setIrk((BigDecimal) rset.getObject("irk"));
			relatorioIndiceIRK.setReceitaMinima((BigDecimal) rset.getObject("receitaminima"));
			relatorioIndiceIRK.setDiferencaReceitas((BigDecimal) rset.getObject("diferencareceita"));
			lsDadosRelatorio.add(relatorioIndiceIRK);
		}
		return lsDadosRelatorio;
	}

}
