package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioW2ISinteticoBean;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioW2ISintetico extends Relatorio {

	private static Logger log = Logger.getLogger(RelatorioW2ISintetico.class);
	private List<RelatorioW2ISinteticoBean> lsDadosRelatorio;

	public RelatorioW2ISintetico(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();

				lsDadosRelatorio = new ArrayList<RelatorioW2ISinteticoBean>();

				String sql = getSql(parametros);

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);
				ResultSet rset = null;

				stmt.setInt("TIPO_DATA", (Integer) parametros.get("TIPO_DATA"));
				stmt.setTimestamp("DATA_INICIAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
				stmt.setTimestamp("DATA_FINAL", new Timestamp(DateUtil.fimFecha((Date) parametros.get("DATA_FINAL")).getTime()));

				if (parametros.get("EMPRESA_ID") != null)
					stmt.setInt("EMPRESA_ID", (Integer) parametros.get("EMPRESA_ID"));

				if (parametros.get("ORIGEN_ID") != null && parametros.get("ORIGEN_ID") != "" && ((Integer) parametros.get("ORIGEN_ID") != -1))
					stmt.setInt("ORIGEN_ID", (Integer) parametros.get("ORIGEN_ID"));

				if (parametros.get("DESTINO_ID") != null && parametros.get("DESTINO_ID") != "" && ((Integer) parametros.get("DESTINO_ID") != -1))
					stmt.setInt("DESTINO_ID", (Integer) parametros.get("DESTINO_ID"));

				rset = stmt.executeQuery();

				RelatorioW2ISinteticoBean relatorioW2ISinteticoBean = null;
				while (rset.next()) {
					relatorioW2ISinteticoBean = new RelatorioW2ISinteticoBean();
					relatorioW2ISinteticoBean.setEmpresaId(rset.getInt("empresaId"));
					relatorioW2ISinteticoBean.setEmpresa(rset.getString("empresa"));
					relatorioW2ISinteticoBean.setPuntoVentaId(rset.getInt("puntoVentaId"));
					relatorioW2ISinteticoBean.setAgencia(rset.getString("agencia"));
					relatorioW2ISinteticoBean.setQtdSeguroOpcionalCancelado(rset.getBigDecimal("qtdSeguroOpcionalCancelado") == null ? BigDecimal.ZERO : rset.getBigDecimal("qtdSeguroOpcionalCancelado"));
					relatorioW2ISinteticoBean.setTotalValorSeguroOpcionalCancelado(rset.getBigDecimal("totalValorSeguroOpcionalCancelado") == null ? BigDecimal.ZERO : rset.getBigDecimal("totalValorSeguroOpcionalCancelado"));
					relatorioW2ISinteticoBean.setQtdSeguroOpcionalVendidos(rset.getBigDecimal("qtdSeguroOpcionalVendidos") == null ? BigDecimal.ZERO : rset.getBigDecimal("qtdSeguroOpcionalVendidos"));
					relatorioW2ISinteticoBean.setTotalValorseguroOpcionalVendido(rset.getBigDecimal("totalValorSeguroOpcionalVendido") == null ? BigDecimal.ZERO : rset.getBigDecimal("totalValorSeguroOpcionalVendido"));
					lsDadosRelatorio.add(relatorioW2ISinteticoBean);

				}

				if (lsDadosRelatorio.size() > 0) {

					setLsDadosRelatorio(lsDadosRelatorio);
				}
			}

		});
	}

	public void setLsDadosRelatorio(List<RelatorioW2ISinteticoBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	protected void processaParametros() throws Exception {
		// TODO Auto-generated method stub

	}

	private String getSql(Map<String, Object> parametros) {
		String puntosVentaIds = (String) parametros.get("NUMPUNTOVENTA");

		StringBuilder sql = new StringBuilder();
		sql.append("SELECT DISTINCT ");
		sql.append("em.empresa_id AS empresaId, ");
		sql.append("em.nombempresa AS empresa, ");
		sql.append("pv.puntoventa_id AS puntoVentaId, ");
		sql.append("pv.nombpuntoventa As agencia, ");
		sql.append(" SUM( (case when sx.status =0 then 1 else 0.0  end)) qtdSeguroOpcionalCancelado, ");
		sql.append(" SUM( (case when sx.status =0 then sx.valor else 0.0 end )) totalValorSeguroOpcionalCancelado, ");
		sql.append(" SUM( (case when sx.status >0 then 1 else 0.0 end )) qtdSeguroOpcionalVendidos, ");
		sql.append(" SUM( (case when sx.status >0 then sx.valor else 0.0 end )) totalValorSeguroOpcionalVendido ");
		sql.append("FROM segpolv sx ");

		sql.append("INNER JOIN boleto b on b.boleto_id=sx.boleto_id ");
		sql.append("INNER JOIN punto_venta pv ON pv.PUNTOVENTA_ID = b.PUNTOVENTA_ID ");
		sql.append("INNER JOIN empresa em ON em.empresa_id=b.empresacorrida_id ");
		sql.append("INNER JOIN parada po ON po.parada_id=b.origen_id ");
		sql.append("INNER JOIN parada pd ON pd.parada_id=b.destino_id ");
		sql.append("INNER JOIN CIUDAD co ON co.ciudad_id=po.ciudad_id ");
		sql.append("INNER JOIN CIUDAD cd ON cd.ciudad_id=pd.ciudad_id ");

		sql.append("WHERE DECODE(:TIPO_DATA, 1, b.FECHORVENTA, 2, b.FECCORRIDA) BETWEEN :DATA_INICIAL AND :DATA_FINAL ");
		if (parametros.get("EMPRESA_ID") != null) {
			sql.append("AND em.empresa_id = COALESCE(:EMPRESA_ID, em.empresa_id) ");
		}
		if (parametros.get("ORIGEN_ID") != null && parametros.get("ORIGEN_ID") != "" && ((Integer) parametros.get("ORIGEN_ID") != -1)) {
			sql.append("AND po.parada_id = COALESCE(:ORIGEN_ID, po.parada_id) ");
		}
		if (parametros.get("DESTINO_ID") != null && parametros.get("DESTINO_ID") != "" && ((Integer) parametros.get("DESTINO_ID") != -1)) {
			sql.append("AND pd.parada_id = COALESCE(:DESTINO_ID, pd.parada_id) ");
		}
		if (!puntosVentaIds.equals("Todas")) {
			sql.append("    AND pv.puntoventa_id IN (").append(puntosVentaIds).append(") ");
		}
		sql.append("GROUP BY em.empresa_id, em.nombempresa,pv.puntoventa_id,pv.nombpuntoventa ");

		return sql.toString();
	}

}
