package com.rjconsultores.ventaboletos.web.utilerias.menu.item.seguridad;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.bind.DatatypeConverter;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
import org.zkoss.zk.ui.util.Clients;
import org.zkoss.zul.Messagebox;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWSAlgorithm;
import com.nimbusds.jose.JWSHeader;
import com.nimbusds.jose.JWSObject;
import com.nimbusds.jose.Payload;
import com.nimbusds.jose.crypto.MACSigner;
import com.nimbusds.jwt.JWTClaimsSet;
import com.rjconsultores.ventaboletos.service.ConstanteService;
import com.rjconsultores.ventaboletos.web.utilerias.menu.DefaultItemMenuSistema;
import com.rjconsultores.ventaboletos.web.utilerias.spring.AppContext;

public class ItemMenuLimparCacheLocalidadesAPI extends DefaultItemMenuSistema {
	private static Logger log = Logger.getLogger(ItemMenuLimparCacheLocalidadesAPI.class);

    private static final String secret = "#KO&Fm4_k.sU9M8`6Mx'F\\\"H:*Qxu]6F4r,)JmZ2Jwafd)I.2[RET'1:)VQ6mG9,";
    private static final int minutesExpireToken = 10;
	
	public ItemMenuLimparCacheLocalidadesAPI() {
		super("indexController.mniLimparCacheLocalidadesAPI.label");
	}

	@Override
	public String getClaveMenu() {
		return "COM.RJCONSULTORES.ADMINISTRACION.GUI.SEGURIDAD.MENU.RECARREGARCACHELOCALIDADESAPI";
	}

	@Override
	public void ejecutar() {
		
		String[] urls = getURLSAPI();
		
		if( urls == null || urls.length == 0 ) {
			Clients.alert(org.zkoss.util.resource.Labels.getLabel("limparCacheLocalidadesAPI.message.naoconfigurado"),
					org.zkoss.util.resource.Labels.getLabel("limparCacheLocalidadesAPI.title"), Messagebox.INFORMATION);
			return;
		}
		
		for (String url : urls) {
			
			if (StringUtils.isBlank(url) || url.contains("|")) {
				Clients.alert(org.zkoss.util.resource.Labels.getLabel("limparCacheLocalidadesAPI.message.naoconfigurado"),
						org.zkoss.util.resource.Labels.getLabel("limparCacheLocalidadesAPI.title"), Messagebox.INFORMATION);
				return;
			}		
			
			url = montarUrlRequest(url);
			httpRequestApi(url);
		}
			
		String message = "Solicitao de limpeza de cache executada com sucesso, o mesmo j est sendo processado pela API.";
		Clients.alert(message, org.zkoss.util.resource.Labels.getLabel("limparCacheLocalidadesAPI.title"), Messagebox.INFORMATION);
	}

	private String montarUrlRequest(String url) {
		
		url = url.toLowerCase();
		
		if (!url.endsWith("/")) {
			url += "/";
		}
		
		return url.concat("adm/limparCache");
	}

	private void httpRequestApi(final String url) {
        new Thread(new Runnable() {
            @Override
            public void run() {
        		
            	HttpClient client = new DefaultHttpClient();
        		HttpGet request = new HttpGet(url);
        		UsernamePasswordCredentials creds = null;
				
        		try {
					creds = new UsernamePasswordCredentials("internal", getToken());
					request.addHeader(new BasicScheme().authenticate(creds, request));
					HttpResponse response = client.execute(request);
					if(response != null && response.getStatusLine() != null) {
						log.info(String.format(":: Response Status Code %d ::", response.getStatusLine().getStatusCode()));
					}
				} catch (Exception e) {
					log.error("Ocorreu um erro ao executar a requisio de limpar cache de localidade da API");
				}
        		
            }
        }).start();
	}

	private String[] getURLSAPI() {
		ApplicationContext appContext = AppContext.getApplicationContext();
		ConstanteService constanteService = (ConstanteService) appContext.getBean("constanteService");
		String constante = constanteService.buscarURLAPI();

		return constante==null ? null : constante.split("\\|"); 
	}

	private String getToken() throws JOSEException {
		long ttlMillis = minutesExpireToken * 60000;
		long nowMillis = System.currentTimeMillis();
		long expMillis = nowMillis + ttlMillis;

		JWTClaimsSet claims = new JWTClaimsSet.Builder()
				.claim("sub", new SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format(new Date()))
				.claim("userId", "adm")
				.claim("role", "ROLE_TOKEN")
				.claim("exp", expMillis)
				.build();

		JWSObject jwsObject = new JWSObject(new JWSHeader(JWSAlgorithm.HS256),
				new Payload(claims.toJSONObject()));

		jwsObject.sign(new MACSigner(DatatypeConverter.parseBase64Binary(secret)));

		return jwsObject.serialize();
	}
}
